"""
Created on 09/07/2014

@author: synerty
"""
from twisted.internet import defer
from twisted.internet.defer import (
    Deferred,
    DeferredList,
    succeed,
    inlineCallbacks,
)
from vortex.DeferUtil import vortexLogFailure
from vortex.Payload import Payload
from vortex.PayloadEndpoint import PayloadEndpoint
from vortex.PayloadEnvelope import PayloadEnvelope
from vortex.VortexFactory import VortexFactory

from peek_plugin_base.PeekVortexUtil import peekServerName

__author__ = "peek"

import logging

logger = logging.getLogger(__name__)

# -------------------------------------
# Software Update Handler for data from agents

# The filter we listen on
peekPlatformVersionFilt = {
    "plugin": "peak_platform",
    "key": "peek_platform.version.check",
}  # LISTEN / SEND


class PeekSwVersionPollHandler(object):
    PEEK_PLATFORM = "peek_platform"

    def __init__(self):
        self._startupDeferred = Deferred()
        self._ep = None

    def start(self):
        from peek_platform import PeekPlatformConfig

        if not PeekPlatformConfig.config.autoPackageUpdate:
            return succeed(True)

        self._ep = PayloadEndpoint(peekPlatformVersionFilt, self._process)

        d = VortexFactory.sendVortexMsg(
            vortexMsgs=Payload(filt=peekPlatformVersionFilt)
            .makePayloadEnvelope()
            .toVortexMsg(),
            destVortexName=peekServerName,
        )
        d.addErrback(vortexLogFailure, logger, consumeError=True)

        return self._startupDeferred

    def _process(self, payloadEnvelope: PayloadEnvelope, **kwargs):
        logger.info(payloadEnvelope.result)
        assert not payloadEnvelope.result  # result is None means success

        from peek_platform import PeekPlatformConfig

        platformUpdate = False
        deferredList = []

        for swVersionInfo in payloadEnvelope.tuples:
            if swVersionInfo.name == self.PEEK_PLATFORM:
                if (
                    PeekPlatformConfig.config.platformVersion
                    != swVersionInfo.version
                ):
                    logger.info(
                        "Recieved platform update new version is %s, we're %s",
                        swVersionInfo.version,
                        PeekPlatformConfig.config.platformVersion,
                    )
                    d = PeekPlatformConfig.peekSwInstallManager.update(
                        swVersionInfo.version
                    )
                    deferredList.append(d)

                    # Don't process any more, we'll update them when we restart.
                    platformUpdate = True
                    break

            else:
                installedPluginVer = PeekPlatformConfig.config.pluginVersion(
                    swVersionInfo.name
                )

                if installedPluginVer != swVersionInfo.version:
                    logger.info(
                        "Recieved %s update new version is %s, we're %s",
                        swVersionInfo.name,
                        swVersionInfo.version,
                        installedPluginVer,
                    )

                    d = PeekPlatformConfig.pluginSwInstallManager.update(
                        swVersionInfo.name, swVersionInfo.version
                    )
                    deferredList.append(d)

        if not self._startupDeferred:
            return

        if deferredList:
            if platformUpdate:
                self._startupDeferred.errback(
                    Exception("Startup stopped, Platform update and restart")
                )
            else:
                DeferredList(deferredList).chainDeferred(self._startupDeferred)

        else:
            logger.info("No update required")
            if self._startupDeferred:
                self._startupDeferred.callback(True)

        self._startupDeferred = None


peekSwVersionPollHandler = PeekSwVersionPollHandler()



import logging

from jsoncfg.value_mappers import require_bool

logger = logging.getLogger(__name__)


class PeekFileConfigDocBuildMixin:
    @property
    def docBuildPrepareEnabled(self) -> bool:
        """Doc Prepare Enabled

        :return True If peek should prepare the build directory

        """
        with self._cfg as c:
            return c.frontend.docBuildPrepareEnabled(True, require_bool)

    @property
    def docSyncFilesForDebugEnabled(self) -> bool:
        """Sync Files for Debug Enabled

        :return True If peek should watch the build files and sync them as they change

        """
        with self._cfg as c:
            return c.frontend.docSyncFilesForDebugEnabled(False, require_bool)

    @property
    def docBuildEnabled(self) -> bool:
        """Doc Build Enabled

        :return True If peek should automatically try to build the docs.

        """
        with self._cfg as c:
            return c.frontend.docBuildEnabled(True, require_bool)



import logging
import multiprocessing
import platform
from typing import Optional

from jsoncfg.value_mappers import require_integer
from jsoncfg.value_mappers import require_string

from peek_platform.WindowsPatch import isMacOS

logger = logging.getLogger(__name__)


class PeekFileConfigWorkerMixin:
    @property
    def taskValkeyUrl(self) -> str:
        if platform.system() == "Darwin":
            default = "valkey://localhost:6379/0"
        else:
            default = "unix:///peek/run/valkey/valkey.sock?db=0"

        with self._cfg as c:
            return c.task.valkey.url(default, require_string)

    @property
    def taskWorkerCount(self) -> int:
        # for CELERYD_CONCURRENCY

        # By default, we assume a single server setup.
        # So leave half the CPU threads for the database
        default = multiprocessing.cpu_count()
        with self._cfg as c:
            return c.task.worker.count(default, require_integer)

    @property
    def taskReplaceWorkerAfterTasksCompleted(self) -> Optional[int]:
        # for worker_max_tasks_per_child
        default = 5000
        with self._cfg as c:
            try:
                return int(
                    c.task.worker.replaceAfterTasksCompleted(
                        default, require_integer
                    )
                )
            except (TypeError, ValueError):
                c.task.worker.replaceAfterTasksCompleted = default
                return default

    @property
    def taskReplaceWorkerAfterSeconds(self) -> Optional[int]:
        # for worker_max_tasks_per_child
        default = 1800
        with self._cfg as c:
            try:
                return int(
                    c.task.worker.replaceAfterTaskSeconds(
                        default, require_integer
                    )
                )
            except (TypeError, ValueError):
                c.task.worker.replaceAfterTaskSeconds = default
                return default

    @property
    def taskReplaceWorkerAfterMemUsage(self) -> Optional[int]:
        # for worker_max_memory_per_child
        default = 500
        with self._cfg as c:
            try:
                return int(
                    c.task.worker.replaceAfterMemMbUsage(
                        default, require_integer
                    )
                )
            except (TypeError, ValueError):
                c.task.worker.replaceAfterMemMbUsage = default
                return default



"""
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
 * Website : http://www.synerty.com
 * Support : support@synerty.com
 *
"""

import logging
import os
import platform
from abc import ABCMeta
from pathlib import Path

from jsoncfg.functions import ConfigWithWrapper
from jsoncfg.functions import save_config

logger = logging.getLogger(__name__)

PEEK_LOGIC_SERVICE = "peek-logic-service"
PEEK_WORKER_SERVICE = "peek-worker-service"
PEEK_AGENT_SERVICE = "peek-agent-service"
PEEK_OFFICE_SERVICE = "peek-office-service"
PEEK_FIELD_SERVICE = "peek-field-service"
PEEK_SERVICES = (
    PEEK_LOGIC_SERVICE,
    PEEK_WORKER_SERVICE,
    PEEK_AGENT_SERVICE,
    PEEK_OFFICE_SERVICE,
    PEEK_FIELD_SERVICE,
)


class PeekFileConfigABC(metaclass=ABCMeta):
    """
    This class creates a basic agent configuration
    """

    DEFAULT_FILE_CHMOD = 0o600
    DEFAULT_DIR_CHMOD = 0o700

    __instance = None

    def __new__(cls):
        if cls.__instance is not None:
            return cls.__instance

        self = super(PeekFileConfigABC, cls).__new__(cls)
        cls.__instance = self
        return self

    def _migrate(self):
        pass

    def __init__(self):
        """
        Constructor
        """
        from peek_platform import PeekPlatformConfig

        assert PeekPlatformConfig.componentName is not None

        serviceNameNoService = PeekPlatformConfig.componentName.replace(
            "-service", ""
        )
        basePath = (
            Path("~/peek").expanduser()
            if platform.system() == "Darwin"
            else Path("~").expanduser()
        )
        self._configPath = str(basePath / "etc" / serviceNameNoService)
        self._sslPath = str(basePath / "etc" / "ssl" / serviceNameNoService)
        self._dataPath = str(basePath / "var" / serviceNameNoService)
        self._logPath = str(basePath / "log" / serviceNameNoService)
        self._tmpPath = str(basePath / "tmp" / serviceNameNoService)

        self._migrate()

        for path in (
            self._configPath,
            self._dataPath,
            self._logPath,
            self._tmpPath,
        ):
            if not os.path.isdir(path):
                assert not os.path.exists(path)
                Path(path).mkdir(
                    mode=self.DEFAULT_DIR_CHMOD, parents=True, exist_ok=True
                )

        self._configFilePath = os.path.join(self._configPath, "config.json")

        if not os.path.isfile(self._configFilePath):
            assert not os.path.exists(self._configFilePath)
            with open(self._configFilePath, "w") as fobj:
                fobj.write("{}")

        self._cfg = ConfigWithWrapper(self._configFilePath)

        self._hp = "%(" + self._configPath + ")s"

    def _save(self):
        save_config(self._configFilePath, self._cfg)

    def _chkDir(self, path):
        if not os.path.isdir(path):
            assert not os.path.exists(path)
            os.makedirs(path, self.DEFAULT_DIR_CHMOD)
        return path



import logging
import uuid
from typing import List
from typing import Optional
from typing import Union

from twisted.internet._posixstdio import StandardIO
from twisted.internet.defer import inlineCallbacks
from vortex.PayloadEnvelope import VortexMsgList
from vortex.VortexABC import VortexABC
from vortex.VortexABC import VortexInfo


logger = logging.getLogger("child_vortex")


class PluginSubprocChildVortex(VortexABC):
    def __init__(self, vortexName: str):
        self._localVortexInfo = VortexInfo(vortexName, uuid.uuid4())
        self._remoteVortexInfos = []
        self._remoteVortexUuidsSet = set()
        self._sendVortexToParentProtocol = None

    def setStdoutProtocol(self, sendVortexToParentProtocol: StandardIO):
        self._sendVortexToParentProtocol = sendVortexToParentProtocol

    @property
    def localVortexInfo(self) -> VortexInfo:
        return self._localVortexInfo

    @property
    def remoteVortexInfo(self) -> List[VortexInfo]:
        return self._remoteVortexInfos

    @property
    def requiresBase64Encoding(self) -> bool:
        return False

    @inlineCallbacks
    def sendVortexMsg(
        self,
        vortexMsgs: Union[VortexMsgList, bytes],
        vortexUuid: Optional[str] = None,
    ):
        if vortexUuid not in self._remoteVortexUuidsSet:
            logger.debug(
                "Remote client vortex with UUID %s doesn't exist", vortexUuid
            )
            return

        if not isinstance(vortexMsgs, list):
            vortexMsgs = [vortexMsgs]

        yield self._sendVortexToParentProtocol.sendVortexMsg(
            vortexMsgs=vortexMsgs, vortexUuid=vortexUuid
        )

    def ensureRemoteVortexIsRegistered(self, vortexUuid: str, vortexName: str):
        if vortexUuid in self._remoteVortexUuidsSet:
            return

        self._remoteVortexInfos.append(
            VortexInfo(name=vortexName, uuid=vortexUuid)
        )
        self._remoteVortexUuidsSet.add(vortexUuid)

    def updateRemoteVortexUuids(self, vortexInfos: list[VortexInfo]):
        self._remoteVortexInfos = vortexInfos
        self._remoteVortexUuidsSet = set([vi.uuid for vi in vortexInfos])



import json
import logging
from base64 import b64decode
from base64 import b64encode
from typing import Union

from twisted.internet import protocol
from twisted.internet.defer import inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.PayloadEnvelope import VortexMsgList
from vortex.PayloadIO import PayloadIO
from vortex.PayloadPriority import DEFAULT_PRIORITY

from peek_platform.subproc_plugin_init.plugin_subproc.plugin_subproc_child_vortex import (
    PluginSubprocChildVortex,
)
from peek_platform.subproc_plugin_init.plugin_subproc.plugin_subproc_vortex_msg_tuple import (
    PluginSubprocVortexMsgTuple,
)
from peek_platform.subproc_plugin_init.plugin_subproc.plugin_subproc_vortex_payload_envelope_tuple import (
    PluginSubprocVortexPayloadEnvelopeTuple,
)


logger = logging.getLogger("child_vortex_protocol")


class PluginSubprocChildVortexProtocol(protocol.Protocol):
    def __init__(self, vortex: PluginSubprocChildVortex):
        self._data = b""
        self._vortex = vortex

    @inlineCallbacks
    def dataReceived(self, data: bytes):
        self._data += data

        while b"." in self._data:
            message, self._data = self._data.split(b".", 1)
            if not message:
                continue

            vortexPayloadTuple = yield self._decodeVortexPayloadTuple(message)

            # assign these locally so they are not used in the closure.
            payloadEnvelope = vortexPayloadTuple.payloadEnvelope
            vortexUuid = vortexPayloadTuple.vortexUuid
            vortexName = vortexPayloadTuple.vortexName
            del vortexPayloadTuple

            self._vortex.ensureRemoteVortexIsRegistered(
                vortexUuid=vortexUuid, vortexName=vortexName
            )

            def sendResponse(
                vortexMsgs: Union[VortexMsgList, bytes],
                priority: int = DEFAULT_PRIORITY,
            ):
                return self.sendVortexMsg(
                    vortexMsgs=vortexMsgs,
                    vortexUuid=vortexUuid,
                    priority=priority,
                )

            PayloadIO().process(
                payloadEnvelope=payloadEnvelope,
                vortexUuid=vortexUuid,
                vortexName=vortexName,
                httpSession=None,
                sendResponse=sendResponse,
            )

    @inlineCallbacks
    def sendVortexMsg(
        self,
        vortexMsgs: Union[VortexMsgList, bytes],
        vortexUuid: str,
        priority: int = DEFAULT_PRIORITY,
    ):
        tuple_ = PluginSubprocVortexMsgTuple(
            vortexUuid=vortexUuid, vortexMsgs=vortexMsgs, priority=priority
        )
        vortexMsgTuple = yield self._encodeVortexMsgTuple(tuple_)
        self.transport.write(vortexMsgTuple)
        self.transport.write(b".")

    @deferToThreadWrapWithLogger(logger)
    def _decodeVortexPayloadTuple(self, message):
        vortexPayloadTuple = (
            PluginSubprocVortexPayloadEnvelopeTuple().fromJsonDict(
                json.loads(b64decode(message))
            )
        )
        return vortexPayloadTuple

    @deferToThreadWrapWithLogger(logger)
    def _encodeVortexMsgTuple(self, tuple_):
        return b64encode(json.dumps(tuple_.toJsonDict()).encode())



import json
import logging
from base64 import b64decode

from twisted.internet import protocol
from twisted.internet.defer import inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.VortexABC import VortexInfo

from peek_platform.subproc_plugin_init.plugin_subproc.plugin_subproc_child_vortex import (
    PluginSubprocChildVortex,
)


logger = logging.getLogger("child_vortex_uuid_protocol")


class PluginSubprocChildVortexUuidProtocol(protocol.Protocol):
    def __init__(self, vortex: PluginSubprocChildVortex):
        self._data = b""
        self._vortex = vortex

    @inlineCallbacks
    def dataReceived(self, data: bytes):
        self._data += data
        if not self._vortex:
            return

        while b"." in self._data:
            message, self._data = self._data.split(b".", 1)
            if not message:
                continue

            # Tell the child fake vortex about the remote UUIDs in our parent
            # process
            vortexInfoLists = yield self._decodeMessage(message)
            self._vortex.updateRemoteVortexUuids(
                [VortexInfo(name=o[0], uuid=o[1]) for o in vortexInfoLists]
            )

    @deferToThreadWrapWithLogger(logger)
    def _decodeMessage(self, message):
        return json.loads(b64decode(message))



import logging

from setproctitle import setproctitle
from twisted.internet import defer
from twisted.internet import reactor
from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.VortexFactory import VortexFactory

from peek_plugin_base.PeekVortexUtil import peekAgentName
from peek_plugin_base.PeekVortexUtil import peekFieldName
from peek_plugin_base.PeekVortexUtil import peekOfficeName
from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_plugin_base.PeekVortexUtil import peekVortexClientServiceNames
from peek_plugin_base.PeekVortexUtil import peekWorkerName


class InitPlatform:
    def __init__(self, serviceName: str, isPluginSubprocess: bool = False):
        self._serviceName = serviceName

        from peek_platform import PeekPlatformConfig

        PeekPlatformConfig.isPluginSubprocess = isPluginSubprocess

        PeekPlatformConfig.componentName = self._serviceName
        if not isPluginSubprocess:
            setproctitle(PeekPlatformConfig.componentName)

    def setupPlatform(self):
        self.setupInstallManagers()
        self.setupPluginLoader()
        self.setupConfig()
        self.setupLogging()
        self.setupMemoryDebugLogging()
        self.setupTwistedReactor()
        self.setupTempDirs()
        self.setupManhole()

    @inlineCallbacks
    def loadAndStartupPlugins(self):
        from peek_platform import PeekPlatformConfig

        yield PeekPlatformConfig.pluginLoader.loadCorePlugins()
        yield PeekPlatformConfig.pluginLoader.loadOptionalPlugins()
        yield PeekPlatformConfig.pluginLoader.startCorePlugins()
        yield PeekPlatformConfig.pluginLoader.startOptionalPlugins()

    @inlineCallbacks
    def stopAndShutdownPluginsAndVortex(self):
        from peek_platform import PeekPlatformConfig

        yield PeekPlatformConfig.pluginLoader.stopOptionalPlugins()
        yield PeekPlatformConfig.pluginLoader.stopCorePlugins()
        yield PeekPlatformConfig.pluginLoader.unloadOptionalPlugins()
        yield PeekPlatformConfig.pluginLoader.unloadCorePlugins()

        yield VortexFactory.shutdown()

    def setupManhole(self):
        from peek_platform import PeekPlatformConfig

        # Setup manhole
        if PeekPlatformConfig.config.manholeEnabled:
            from peek_platform.util.ManHoleUtil import start_manhole

            start_manhole(
                PeekPlatformConfig.config.manholePort,
                PeekPlatformConfig.config.manholePassword,
                PeekPlatformConfig.config.manholePublicKeyFile,
                PeekPlatformConfig.config.manholePrivateKeyFile,
            )

    def setupTempDirs(self):
        from peek_platform import PeekPlatformConfig

        # Initialise the txhttputil Directory object
        from pytmpdir.dir_setting import DirSetting

        DirSetting.defaultDirChmod = PeekPlatformConfig.config.DEFAULT_DIR_CHMOD
        DirSetting.tmpDirPath = PeekPlatformConfig.config.tmpPath

        from txhttputil.site.FileUploadRequest import FileUploadRequest

        FileUploadRequest.tmpFilePath = PeekPlatformConfig.config.tmpPath

    def setupTwistedReactor(self):
        from peek_platform import PeekPlatformConfig

        # Set the reactor thread count
        reactor.suggestThreadPoolSize(
            PeekPlatformConfig.config.twistedThreadPoolSize
        )

    def setupMemoryDebugLogging(self):
        from peek_platform import PeekPlatformConfig

        # If we need to enable memory debugging, turn that on.
        if PeekPlatformConfig.config.loggingDebugMemoryMask:
            from peek_platform.util.MemUtil import setupMemoryDebugging

            setupMemoryDebugging(
                PeekPlatformConfig.componentName,
                PeekPlatformConfig.config.loggingDebugMemoryMask,
            )

    def setupLogging(self):
        from peek_platform import PeekPlatformConfig

        # Set default logging level
        logging.root.setLevel(PeekPlatformConfig.config.loggingLevel)
        from peek_platform.util.LogUtil import updatePeekLoggerHandlers

        # PsUtil
        if not PeekPlatformConfig.config.loggingLogSystemMetrics:
            logging.getLogger("peek_plugin_base.util.PeekPsUtil").setLevel(999)

        updatePeekLoggerHandlers(
            PeekPlatformConfig.componentName,
            PeekPlatformConfig.config.daysToKeep,
            PeekPlatformConfig.config.logToStdout,
        )
        if PeekPlatformConfig.config.loggingLogToSyslogHost:
            from peek_platform.util.LogUtil import setupLoggingToSyslogServer

            setupLoggingToSyslogServer(
                PeekPlatformConfig.config.loggingLogToSyslogHost,
                PeekPlatformConfig.config.loggingLogToSyslogPort,
                PeekPlatformConfig.config.loggingLogToSyslogFacility,
            )
        # Enable deferred debugging if DEBUG is on.
        if logging.root.level == logging.DEBUG:
            defer.setDebugging(True)

    def setupConfig(self):
        from peek_platform import PeekPlatformConfig

        if self._serviceName == peekOfficeName:
            # The config depends on the componentName, order is important
            from peek_office_service.PeekClientConfig import (
                PeekClientConfig as Config,
            )
            from peek_office_service import __version__

        elif self._serviceName == peekFieldName:
            # The config depends on the componentName, order is important
            from peek_field_service.PeekClientConfig import (
                PeekClientConfig as Config,
            )
            from peek_field_service import __version__

        elif self._serviceName == peekServerName:
            # The config depends on the componentName, order is important
            from peek_logic_service.PeekServerConfig import (
                PeekServerConfig as Config,
            )
            from peek_logic_service import __version__

        elif self._serviceName == peekAgentName:
            # The config depends on the componentName, order is important
            from peek_agent_service.PeekAgentConfig import (
                PeekAgentConfig as Config,
            )
            from peek_agent_service import __version__

        elif self._serviceName == peekWorkerName:
            # The config depends on the componentName, order is important
            from peek_worker_service.PeekWorkerConfig import (
                PeekWorkerConfig as Config,
            )
            from peek_worker_service import __version__

        else:
            raise NotImplementedError()

        PeekPlatformConfig.config = Config()
        PeekPlatformConfig.config.platformVersion = __version__

    def setupPluginLoader(self):
        from peek_platform import PeekPlatformConfig

        if self._serviceName == peekOfficeName:
            # Tell the platform classes about our instance of the PeekLoaderBase
            from peek_office_service.plugin.ClientPluginLoader import (
                ClientPluginLoader as Loader,
            )

        elif self._serviceName == peekFieldName:
            # Tell the platform classes about our instance of the PeekLoaderBase
            from peek_field_service.plugin.ClientPluginLoader import (
                ClientPluginLoader as Loader,
            )

        elif self._serviceName == peekServerName:
            # Tell the platform classes about our instance of the PeekLoaderBase
            from peek_logic_service.plugin.ServerPluginLoader import (
                ServerPluginLoader as Loader,
            )

        elif self._serviceName == peekAgentName:
            # Tell the platform classes about our instance of the PeekLoaderBase
            from peek_agent_service.plugin.AgentPluginLoader import (
                AgentPluginLoader as Loader,
            )

        elif self._serviceName == peekWorkerName:
            # Tell the platform classes about our instance of the PeekLoaderBase
            from peek_worker_service.plugin.WorkerPluginLoader import (
                WorkerPluginLoader as Loader,
            )

        else:
            raise NotImplementedError()

        PeekPlatformConfig.pluginLoader = Loader()

    def setupInstallManagers(self):
        from peek_platform import PeekPlatformConfig

        if self._serviceName == peekOfficeName:
            from peek_office_service.sw_install.PluginSwInstallManager import (
                PluginSwInstallManager,
            )
            from peek_office_service.sw_install.PeekSwInstallManager import (
                PeekSwInstallManager,
            )

        elif self._serviceName == peekFieldName:
            from peek_field_service.sw_install.PluginSwInstallManager import (
                PluginSwInstallManager,
            )
            from peek_field_service.sw_install.PeekSwInstallManager import (
                PeekSwInstallManager,
            )

        elif self._serviceName == peekServerName:
            from peek_logic_service.sw_install.PluginSwInstallManager import (
                PluginSwInstallManager,
            )
            from peek_logic_service.sw_install.PeekSwInstallManager import (
                PeekSwInstallManager,
            )

        elif self._serviceName == peekAgentName:
            from peek_agent_service.sw_install.PluginSwInstallManager import (
                PluginSwInstallManager,
            )
            from peek_agent_service.sw_install.PeekSwInstallManager import (
                PeekSwInstallManager,
            )

        elif self._serviceName == peekWorkerName:
            from peek_worker_service.sw_install.PluginSwInstallManager import (
                PluginSwInstallManager,
            )
            from peek_worker_service.sw_install.PeekSwInstallManager import (
                PeekSwInstallManager,
            )

        else:
            raise NotImplementedError()

        PeekPlatformConfig.pluginSwInstallManager = PluginSwInstallManager()
        PeekPlatformConfig.peekSwInstallManager = PeekSwInstallManager()

    def connectVortexClient(self) -> Deferred:
        assert (
            self._serviceName in peekVortexClientServiceNames
        ), f"Service {self._serviceName} is not meant to connect vortex clients"

        # First, setup the VortexServer Agent
        from peek_platform import PeekPlatformConfig

        dataExchangeCfg = PeekPlatformConfig.config.dataExchange

        scheme = "wss" if dataExchangeCfg.peekServerUseSSL else "ws"
        host = dataExchangeCfg.peekServerHost
        port = dataExchangeCfg.peekServerHttpPort

        return VortexFactory.createWebsocketClient(
            PeekPlatformConfig.componentName,
            host,
            port,
            url=f"{scheme}://{host}:{port}/vortexws",
            sslEnableMutualTLS=dataExchangeCfg.peekServerSSLEnableMutualTLS,
            sslClientCertificateBundleFilePath=dataExchangeCfg.peekServerSSLClientBundleFilePath,
            sslMutualTLSCertificateAuthorityBundleFilePath=dataExchangeCfg.peekServerSSLClientMutualTLSCertificateAuthorityBundleFilePath,
            sslMutualTLSTrustedPeerCertificateBundleFilePath=dataExchangeCfg.peekServerSSLMutualTLSTrustedPeerCertificateBundleFilePath,
        )



from abc import ABCMeta

from jsoncfg.value_mappers import require_string, require_integer
from peek_platform.WindowsPatch import isWindows


class PeekFileConfigOsMixin(metaclass=ABCMeta):
    _bashDefault = (
        "C:\\Program Files\\Git\\bin\\bash.exe" if isWindows else "/bin/bash"
    )

    ### SERVER SECTION ###
    @property
    def bashLocation(self):
        """Bash Location

        :return: The location of the bash interpreter

        :windows: "C:\Program Files\Git\bin\bash.exe"
        :Linux: /bin/bash

        """
        with self._cfg as c:
            return c.os.bashLocation(self._bashDefault, require_string)
